﻿using System;
using System.Collections.Generic;
// using System.Linq;
using System.Text;
using System.Data.OracleClient;
using System.Data.Common;
using System.Data;


namespace AdoTestConsole
{
    public class Author
    {        
        int id;

        public int Id
        {
            get { return id; }
            set { id = value; }
        }
        string name;

        public string Name
        {
            get { return name; }
            set { name = value; }
        }
        
        DateTime birthDate;

        public DateTime BirthDate
        {
            get { return birthDate; }
            set { birthDate = value; }
        }

        public void Save()
        {
            Author.Save(this);
        }

        public void Delete()
        {
            Author.Delete(this);
        }

        public static List<Author> Find(string likeName)
        {
            OracleConnection connection = new OracleConnection(Resources.connectionString);
            connection.Open();

            OracleCommand listCommand = new OracleCommand("SELECT * from AUTHOR where NAME LIKE :likeName", connection);
            listCommand.Parameters.Add(":likeName", OracleType.VarChar);
            listCommand.Parameters[":likeName"].Value = "%" + likeName + "%";
            OracleDataReader dataReader = listCommand.ExecuteReader();

            List<Author> authors = new List<Author>();
            while (dataReader.Read())
            {
                Author author = new Author();
                author.Id = dataReader.GetInt32(0);
                author.Name = dataReader.GetString(1);
                author.BirthDate = dataReader.GetDateTime(2);
                authors.Add(author);
            }

            connection.Close();
            return authors;
        }

        public static Author Load(int id)
        {
            OracleConnection connection = new OracleConnection(Resources.connectionString);
            connection.Open();

            OracleCommand listCommand = new OracleCommand("SELECT * from AUTHOR where id=:id", connection);
            listCommand.Parameters.Add(":id", OracleType.Int32);
            listCommand.Parameters[":id"].Value = id;
            OracleDataReader dataReader = listCommand.ExecuteReader();

            Author author = null;

            if (dataReader.Read())
            {
                author = new Author();
                author.Id = dataReader.GetInt32(0);
                author.Name = dataReader.GetString(1);
                author.BirthDate = dataReader.GetDateTime(2);;
            }

            connection.Close();
            return author;
        }

        public static void Save(Author author)
        {
            Author storedAuthor = Load(author.Id);

            OracleConnection connection = new OracleConnection(Resources.connectionString);            
            try
            {
                connection.Open();
                OracleCommand saveCommand;
                
                if (storedAuthor == null)
                {
                    saveCommand = new OracleCommand("INSERT INTO AUTHOR VALUES(:id,:name,:birthdayDate)", connection);
                }
                else
                {
                    saveCommand = new OracleCommand("UPDATE AUTHOR SET NAME=:name,BIRTHDATE=:birthdayDate where ID=:id", connection);
                }

                
                saveCommand.Parameters.Add(":id", OracleType.Int32);
                saveCommand.Parameters.Add(":name", OracleType.VarChar);
                saveCommand.Parameters.Add(":birthdayDate", OracleType.DateTime);

                saveCommand.Parameters[":id"].Value = author.Id;
                saveCommand.Parameters[":name"].Value = author.Name;
                saveCommand.Parameters[":birthdayDate"].Value = author.BirthDate;

                try
                {
                    int affectedRecords = saveCommand.ExecuteNonQuery();
                    Console.WriteLine("{0} rows affected", affectedRecords);
                }
                catch (OracleException ex)
                {
                    Console.WriteLine(ex.Message);
                    //throw ex; 
                }
            }
            finally
            {
                connection.Close();
            }                       
        }

        public static void Delete(Author author)
        {
            OracleConnection connection = new OracleConnection(Resources.connectionString);
            try
            {
                connection.Open();

                OracleCommand deleteCommand = new OracleCommand("DELETE FROM AUTHOR WHERE ID=:id", connection);
                deleteCommand.Parameters.Add(":id", OracleType.Int32);
                deleteCommand.Parameters[":id"].Value = author.Id;

                try
                {
                    int affectedRecords = deleteCommand.ExecuteNonQuery();
                    Console.WriteLine("{0} rows affected", affectedRecords);
                }
                catch (OracleException ex)
                {
                    Console.WriteLine(ex.Message);
                    //throw ex; 
                }
            }
            finally
            {
                connection.Close();
            }
        }
    }
}
