﻿using System;
using System.Collections.Generic;
// using System.Linq;
using System.Text;
using System.Data.OracleClient;
using System.Data.Common;
using System.Data;

namespace AdoTestConsole
{
    public class AuthorsGateway
    {
       
        public void ListAuthors()
        {
            OracleConnection connection = new OracleConnection(Resources.connectionString);
            OracleCommand listCommand = new OracleCommand("SELECT * from AUTHOR order by id", connection);
            connection.Open();

            OracleDataReader dataReader = listCommand.ExecuteReader();

            Console.WriteLine("ID\tNAME\tBIRTHDAY DATE");
            Console.WriteLine("=====================================================");    
            
            while (dataReader.Read())
            {                
                int id = dataReader.GetInt32(0);
                string name = dataReader.GetString(1);
                DateTime birthdayDate = dataReader.GetDateTime(2);
                Console.WriteLine(String.Format("{0}\t{1}\t{2}",id,name,birthdayDate));
            }

            connection.Close();
        }

        public void AddNewAuthor(int id, string name, DateTime birthdayDate)
        {
            OracleConnection connection = new OracleConnection(Resources.connectionString);
            try
            {
                connection.Open();

                OracleCommand insertCommand = new OracleCommand("INSERT INTO AUTHOR VALUES(:id,:name,:birthdayDate)", connection);
                insertCommand.Parameters.Add(":id", OracleType.Int32);
                insertCommand.Parameters.Add(":name", OracleType.VarChar);
                insertCommand.Parameters.Add(":birthdayDate", OracleType.DateTime);


                insertCommand.Parameters[":id"].Value = id;
                insertCommand.Parameters[":name"].Value = name;
                insertCommand.Parameters[":birthdayDate"].Value = birthdayDate;

                try
                {
                    int affectedRecords = insertCommand.ExecuteNonQuery();
                    Console.WriteLine("{0} rows affected", affectedRecords);
                }
                catch (OracleException ex)
                {
                    Console.WriteLine(ex.Message);
                    throw ex; 
                }
            }
            finally
            {
                connection.Close();
            }
        }

        public int AddNewAuthorViaSP(string name, DateTime birthdayDate)
        {
            OracleConnection connection = new OracleConnection(Resources.connectionString);
            try
            {
                connection.Open();

                OracleCommand insertCommand = new OracleCommand("ADD_AUTHOR", connection);
                insertCommand.CommandType = CommandType.StoredProcedure;
                insertCommand.Parameters.Add("p_id", OracleType.Int32);
                insertCommand.Parameters.Add("p_name", OracleType.VarChar);
                insertCommand.Parameters.Add("p_BIRTHDATE", OracleType.DateTime);

                insertCommand.Parameters["p_id"].Direction = ParameterDirection.Output;
                insertCommand.Parameters["p_name"].Value = name;
                insertCommand.Parameters["p_BIRTHDATE"].Value = birthdayDate;               
                insertCommand.ExecuteNonQuery();
                int insertedId = (int)insertCommand.Parameters["p_id"].Value;
                return insertedId; 
            }
            finally
            {
                connection.Close();
            }
            return -1;
        }



        public void removeAuthor(int id)
        {
            OracleConnection connection = new OracleConnection(Resources.connectionString);
            try
            {
                connection.Open();

                OracleCommand deleteCommand = new OracleCommand("DELETE FROM AUTHOR WHERE ID=:id", connection);
                deleteCommand.Parameters.Add(":id", OracleType.Int32);               
                deleteCommand.Parameters[":id"].Value = id;
                
                try
                {
                    int affectedRecords = deleteCommand.ExecuteNonQuery();
                    Console.WriteLine("{0} rows affected", affectedRecords);
                }
                catch (OracleException ex)
                {
                    Console.WriteLine(ex.Message);
                    //throw ex; 
                }
            }
            finally
            {
                connection.Close();
            }
        }

        public void UpdateAuthor(int id, string name, DateTime birthdayDate)
        {
            OracleConnection connection = new OracleConnection(Resources.connectionString);
            try
            {
                connection.Open();

                OracleCommand updateCommand = new OracleCommand("UPDATE AUTHOR SET NAME=:name,BIRTHDATE=:birthdayDate where ID=:id", connection);
                updateCommand.Parameters.Add(":id", OracleType.Int32);
                updateCommand.Parameters.Add(":name", OracleType.VarChar);
                updateCommand.Parameters.Add(":birthdayDate", OracleType.DateTime);

                updateCommand.Parameters[":id"].Value = id;
                updateCommand.Parameters[":name"].Value = name;
                updateCommand.Parameters[":birthdayDate"].Value = birthdayDate;

                try
                {
                    int affectedRecords = updateCommand.ExecuteNonQuery();
                    Console.WriteLine("{0} rows affected", affectedRecords);
                }
                catch (OracleException ex)
                {
                    Console.WriteLine(ex.Message);
                    //throw ex; 
                }
            }
            finally
            {
                connection.Close();
            }
        }


        public void removeAllAuthors()
        {
            OracleConnection connection = new OracleConnection(Resources.connectionString);
            try
            {
                connection.Open();

                OracleCommand deleteCommand = new OracleCommand("DELETE FROM AUTHOR", connection);

                try
                {
                    int affectedRecords = deleteCommand.ExecuteNonQuery();
                    Console.WriteLine("{0} rows affected", affectedRecords);
                }
                catch (OracleException ex)
                {
                    Console.WriteLine(ex.Message);
                    //throw ex; 
                }
            }
            finally
            {
                connection.Close();
            }
        }


        public void TransactionTest()
        {
            OracleConnection connection = new OracleConnection(Resources.connectionString);
            OracleTransaction tx=null;

            try
            {
                connection.Open();
                tx = connection.BeginTransaction();
                OracleCommand deleteCommand = new OracleCommand("DELETE FROM AUTHOR", connection);
                deleteCommand.Transaction = tx;                
                int affectedRecords = deleteCommand.ExecuteNonQuery();
                Console.WriteLine("{0} rows affected", affectedRecords);
                                
                //throw new ApplicationException("transaction interrupted");
                                
                OracleCommand insertCommand = new OracleCommand("INSERT INTO AUTHOR VALUES(:id,:name,:birthdayDate)", connection);
                
                insertCommand.Transaction = tx;
                
                insertCommand.Parameters.Add(":id", OracleType.Int32);
                insertCommand.Parameters.Add(":name", OracleType.VarChar);
                insertCommand.Parameters.Add(":birthdayDate", OracleType.DateTime);

                insertCommand.Parameters[":id"].Value = 11112;
                insertCommand.Parameters[":name"].Value = "!!!";
                insertCommand.Parameters[":birthdayDate"].Value = DateTime.Now;

                insertCommand.ExecuteNonQuery();

                System.Console.WriteLine("COMMIT TRANSACTION");
                tx.Commit();
            }
            catch(Exception ex)
            {
                System.Console.WriteLine(ex.Message);
                System.Console.WriteLine("ROLLBACK TRANSACTION");
                tx.Rollback();
            }
            finally
            {                
                connection.Close();
            }
        }

    }
}
